
-- Localize it for non-English clients.
SUPERPETFRAME_TITLE = "SuperPet";
SUPERPETFRAME_DRAG = "Drag to move";
SUPERPETFRAME_DRAG_LOCKED = "Use /petframe unlock to move.";

-- Packages all local variables of SuperPetFrame Addon.
SuperPetFrameLocalVariables = {};
local l = SuperPetFrameLocalVariables;
local i,j,k;

l.MAX_SUPERPET_DEBUFFS = 16;
l.MAX_SUPERPET_BUFFS = 32;
l.CURRENT_SUPERPET_NUM_DEBUFFS = 0;
l.TARGET_BUFFS_PER_ROW = 8;
l.TARGET_DEBUFFS_PER_ROW = 8;
l.LARGE_BUFF_SIZE = 21;
l.LARGE_BUFF_FRAME_SIZE = 23;
l.SMALL_BUFF_SIZE = 17;
l.SMALL_BUFF_FRAME_SIZE = 19;

l.largeBuffList = {};
l.largeDebuffList = {};

-- Saved variables
SuperPetFrameOptions = SuperPetFrameOptions or {};

SlashCmdList["SUPERPETFRAME"] = function(msg)
  local cmd,var = strsplit(' ', msg or "")
  if cmd == "scale" and tonumber(var) then
    SuperPetFrame_SetScale(var);
  elseif cmd == "reset" then
    SuperPetFrame_Reset();
  elseif cmd == "lock" then
    SuperPetFrameOptions.lockpos = true;
  elseif cmd == "unlock" then
    SuperPetFrameOptions.lockpos = nil;
  else
    DEFAULT_CHAT_FRAME:AddMessage('SuperPetFrame usage:');
    DEFAULT_CHAT_FRAME:AddMessage('/petframe scale <num> : Set scale.');
    DEFAULT_CHAT_FRAME:AddMessage('/petframe reset : Reset position.');
    DEFAULT_CHAT_FRAME:AddMessage('/petframe lock : Lock position.');
    DEFAULT_CHAT_FRAME:AddMessage('/petframe unlock : Unlock position.');
  end
end
SLASH_SUPERPETFRAME1 = "/petframe";

function SuperPetFrame_SetScale(scale)
  if tonumber(scale) then
    SuperPetFrameOptions.scale = tonumber(scale);
    local os = SuperPetFrame:GetScale();
    local ox = SuperPetFrame:GetLeft();
    local oy = SuperPetFrame:GetTop();
    SuperPetFrame:SetScale(scale);
    SuperPetFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", ox*os/scale, oy*os/scale);
  end
end

function SuperPetFrame_Reset()
    SuperPetFrame:SetPoint("TOPLEFT", UIParent, "CENTER", 0, 0);
end

function SuperPetFrame_OnLoad()
	this.statusCounter = 0;
	this.statusSign = -1;
	this.unitHPPercent = 1;

	this.buffStartX = 105;
	this.buffStartY = 32;
	this.buffSpacing = 3;

	SuperPetFrame_Update();
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PET_ATTACK_START");
	this:RegisterEvent("PET_ATTACK_STOP");
	this:RegisterEvent("UNIT_PET");
	this:RegisterEvent("UNIT_COMBAT");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("UNIT_LEVEL");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("VARIABLES_LOADED");

	local frameLevel = SuperPetFrameTextureFrame:GetFrameLevel();
	SuperPetFrameHealthBar:SetFrameLevel(frameLevel-1);
	SuperPetFrameManaBar:SetFrameLevel(frameLevel-1);
	SuperPetFrameSpellBar:SetFrameLevel(frameLevel-1);

	local showmenu = function()
		ToggleDropDownMenu(1, nil, SuperPetFrameDropDown, "SuperPetFrame", 120, 10);
	end
	SecureUnitButton_OnLoad(this, "pet", showmenu);

	RegisterUnitWatch(SuperPetFrame);  
    ClickCastFrames = ClickCastFrames or { };
    ClickCastFrames[this] = true;
end

function SuperPetFrame_Update()
	-- This check is here so the frame will hide when the pet goes away
	-- even if some of the functions below are hooked by addons.
	if ( UnitExists("pet") ) then
		TargetofSuperPet_Update();

		UnitFrame_Update();
		SuperPetFrame_CheckHappiness();
		SuperPetFrame_CheckLevel();
		SuperPetFrame_CheckDead();
		SuperPetDebuffButton_Update();
		SuperPetPortrait:SetAlpha(1.0);

		PetFrame:Hide();

	end
end

function SuperPetFrame_OnEvent(event)
	UnitFrame_OnEvent(event);

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		SuperPetFrame_Update();
	elseif ( event == "UNIT_PET" ) then
		if ( arg1 == "player" ) then
			SuperPetFrame_Update();
		end
	elseif ( event == "UNIT_AURA" ) then
		if ( arg1 == "pet" ) then
			SuperPetDebuffButton_Update();
		end
	elseif ( event == "UNIT_HEALTH" ) then
		if ( arg1 == "pet" ) then
			SuperPetFrame_CheckDead();
		end
	elseif ( event == "UNIT_LEVEL" ) then
		if ( arg1 == "pet" ) then
			SuperPetFrame_CheckLevel();
		end
	elseif ( event == "UNIT_HAPPINESS" ) then
		SuperPetFrame_CheckHappiness();
  	elseif ( event == "VARIABLES_LOADED" ) then
    	SuperPetFrameOptions.scale = SuperPetFrameOptions.scale or 1;
		SuperPetFrame_SetScale(SuperPetFrameOptions.scale);
	end
end

function SuperPetFrame_OnHide()
	PlaySound("INTERFACESOUND_LOSTTARGETUNIT");
	CloseDropDownMenus();
end

function SuperPetFrame_CheckLevel()
	local targetLevel = UnitLevel("pet");
	
	SuperPetLevelText:SetText(targetLevel);
	SuperPetLevelText:SetVertexColor(1.0, 0.82, 0.0);
	SuperPetLevelText:Show();
end

function SuperPetFrame_CheckDead()
	if ( (MobHealthDB == nil) and GetCVar("targetStatusText") == "0" and (UnitHealth("pet") <= 0) and UnitIsConnected("pet") ) then
		SuperPetDeadText:Show();
	else
		SuperPetDeadText:Hide();
	end
end

function SuperPetFrame_OnUpdate()
	if ( TargetofSuperPetFrame:IsShown() ~= UnitExists("pettarget") ) then
		TargetofSuperPet_Update();
	end
end

function SuperPetDebuffButton_Update()
	local button;
	local name, rank, icon, count, duration, timeLeft;
	local buffCount;
	local numBuffs = 0;
--	local largeBuffList = {};
	local playerIsSuperPet = UnitIsUnit("player", "pet");
	local cooldown, startCooldownTime;

	for i=1, l.MAX_SUPERPET_BUFFS do
		name, rank, icon, count, duration, timeLeft = UnitBuff("pet", i);
		button = getglobal("SuperPetFrameBuff"..i);
		if ( not button ) then
			if ( not icon ) then
				break;
			else
				button = CreateFrame("Button", "SuperPetFrameBuff"..i, SuperPetFrame, "SuperPetBuffButtonTemplate");
			end
		end
		
		if ( icon ) then
			getglobal("SuperPetFrameBuff"..i.."Icon"):SetTexture(icon);
			buffCount = getglobal("SuperPetFrameBuff"..i.."Count");
			button:Show();
			if ( count > 1 ) then
				buffCount:SetText(count);
				buffCount:Show();
			else
				buffCount:Hide();
			end
			
			-- Handle cooldowns
			cooldown = getglobal("SuperPetFrameBuff"..i.."Cooldown");
			if ( duration ) then
				if ( duration > 0 ) then
					cooldown:Show();
					startCooldownTime = GetTime()-(duration-timeLeft);
					CooldownFrame_SetTimer(cooldown, startCooldownTime, duration, 1);
				else
					cooldown:Hide();
				end
				
				-- Set the buff to be big if the buff is cast by the player and the pet is not the player
				if ( not playerIsSuperPet ) then
					l.largeBuffList[i] = 1;
				else
					l.largeBuffList[i] = nil;
				end
			else
				cooldown:Hide();
			end

			button.id = i;
			numBuffs = numBuffs + 1; 
			button:ClearAllPoints();
		else
			button:Hide();
		end
	end

	local debuffType, color;
	local debuffCount;
	local numDebuffs = 0;
--	local largeDebuffList = {};
	for i=1, l.MAX_SUPERPET_DEBUFFS do
		local debuffBorder = getglobal("SuperPetFrameDebuff"..i.."Border");
		name, rank, icon, count, debuffType, duration, timeLeft = UnitDebuff("pet", i);
		button = getglobal("SuperPetFrameDebuff"..i);
		if ( not button ) then
			if ( not icon ) then
				break;
			else
				button = CreateFrame("Button", "SuperPetFrameDebuff"..i, SuperPetFrame, "SuperPetDebuffButtonTemplate");
				debuffBorder = getglobal("SuperPetFrameDebuff"..i.."Border");
			end
		end
		if ( icon ) then
			getglobal("SuperPetFrameDebuff"..i.."Icon"):SetTexture(icon);
			debuffCount = getglobal("SuperPetFrameDebuff"..i.."Count");
			if ( debuffType ) then
				color = DebuffTypeColor[debuffType];
			else
				color = DebuffTypeColor["none"];
			end
			if ( count > 1 ) then
				debuffCount:SetText(count);
				debuffCount:Show();
			else
				debuffCount:Hide();
			end

			-- Handle cooldowns
			cooldown = getglobal("SuperPetFrameDebuff"..i.."Cooldown");
			if ( duration  ) then
				if ( duration > 0 ) then
					cooldown:Show();
					startCooldownTime = GetTime()-(duration-timeLeft);
					CooldownFrame_SetTimer(cooldown, startCooldownTime, duration, 1);
				else
					cooldown:Hide();
				end
				-- Set the buff to be big if the buff is cast by the player
				l.largeDebuffList[i] = 1;
			else
				cooldown:Hide();
				l.largeDebuffList[i] = nil;
			end
			
			debuffBorder:SetVertexColor(color.r, color.g, color.b);
			button:Show();
			numDebuffs = numDebuffs + 1;
			button:ClearAllPoints();
		else
			button:Hide();
		end
		button.id = i;
	end
	
	-- Figure out general information that affects buff sizing and positioning
	local numFirstRowBuffs;
	local buffSize = l.LARGE_BUFF_SIZE;
	local buffFrameSize = l.LARGE_BUFF_FRAME_SIZE;
	if ( TargetofSuperPetFrame:IsShown() ) then
		numFirstRowBuffs = 5;
	else
		numFirstRowBuffs = 6;
	end
	if ( getn(l.largeBuffList) > 0 or getn(l.largeDebuffList) > 0 ) then
		numFirstRowBuffs = numFirstRowBuffs - 1;
	end
	-- Shrink the buffs if there are too many of them
	if ( (numBuffs >= numFirstRowBuffs) or (numDebuffs >= numFirstRowBuffs) ) then
		buffSize = l.SMALL_BUFF_SIZE;
		buffFrameSize = l.SMALL_BUFF_FRAME_SIZE;
	end
		
	-- Reset number of buff rows
	SuperPetFrame.buffRows = 0;
	-- Position buffs
	local size;
	local previousWasPlayerCast;
	local offset;
	for i=1, numBuffs do
		if ( l.largeBuffList[i] ) then
			size = l.LARGE_BUFF_SIZE;
			offset = 3;
			previousWasPlayerCast = 1;
		else
			size = buffSize;
			offset = 3;
			if ( previousWasPlayerCast ) then
				offset = 6;
				previousWasPlayerCast = nil;
			end
		end
		SuperPetFrame_UpdateBuffAnchor("SuperPetFrameBuff", i, numFirstRowBuffs, numDebuffs, size, offset, TargetofSuperPetFrame:IsShown());
	end
	-- Position debuffs
	previousWasPlayerCast = nil;
	for i=1, numDebuffs do
		if ( l.largeDebuffList[i] ) then
			size = l.LARGE_BUFF_SIZE;
			offset = 4;
			previousWasPlayerCast = 1;
		else
			size = buffSize;
			offset = 4;
			if ( previousWasPlayerCast ) then
				offset = 6;
				previousWasPlayerCast = nil;
			end
		end
		SuperPetFrame_UpdateDebuffAnchor("SuperPetFrameDebuff", i, numFirstRowBuffs, numBuffs, size, offset, TargetofSuperPetFrame:IsShown());
 	end

	-- update the spell bar position
	SuperPet_Spellbar_AdjustPosition();
end

function SuperPetFrame_UpdateBuffAnchor(buffName, index, numFirstRowBuffs, numDebuffs, buffSize, offset, hasTargetofSuperPet)
	local buff = getglobal(buffName..index);
	
	if ( index == 1 ) then
		if ( UnitIsFriend("player", "pet") ) then
			buff:SetPoint("TOPLEFT", SuperPetFrame, "BOTTOMLEFT", SuperPetFrame.buffStartX, SuperPetFrame.buffStartY);
		else
			if ( numDebuffs > 0 ) then
				buff:SetPoint("TOPLEFT", SuperPetFrameDebuffs, "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
			else
				buff:SetPoint("TOPLEFT", SuperPetFrame, "BOTTOMLEFT", SuperPetFrame.buffStartX, (SuperPetFrame.buffStartY-buffSize-SuperPetFrame.buffSpacing));
			end
		end
		SuperPetFrameBuffs:SetPoint("TOPLEFT", buff, "TOPLEFT", 0, 0);
		SuperPetFrameBuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( index == (numFirstRowBuffs+1) ) then
		buff:SetPoint("TOPLEFT", getglobal(buffName..1), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameBuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( hasTargetofSuperPet and index == (2*numFirstRowBuffs+1) ) then
		buff:SetPoint("TOPLEFT", getglobal(buffName..(numFirstRowBuffs+1)), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameBuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( (index > numFirstRowBuffs) and (mod(index+(l.TARGET_BUFFS_PER_ROW-numFirstRowBuffs), l.TARGET_BUFFS_PER_ROW) == 1) and not hasTargetofSuperPet ) then
		-- Make a new row, have to take the number of buffs in the first row into account
		buff:SetPoint("TOPLEFT", getglobal(buffName..(index-l.TARGET_BUFFS_PER_ROW)), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameBuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	else
		-- Just anchor to previous
		buff:SetPoint("TOPLEFT", getglobal(buffName..(index-1)), "TOPRIGHT", offset, 0);
	end

	-- Resize
	buff:SetWidth(buffSize);
	buff:SetHeight(buffSize);
end

function SuperPetFrame_UpdateDebuffAnchor(buffName, index, numFirstRowBuffs, numBuffs, buffSize, offset, hasTargetofSuperPet)
	local buff = getglobal(buffName..index);

	if ( index == 1 ) then
		if ( UnitIsFriend("player", "pet") and (numBuffs > 0) ) then
			buff:SetPoint("TOPLEFT", SuperPetFrameBuffs, "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		else
			buff:SetPoint("TOPLEFT", SuperPetFrame, "BOTTOMLEFT", SuperPetFrame.buffStartX, SuperPetFrame.buffStartY);
		end
		SuperPetFrameDebuffs:SetPoint("TOPLEFT", buff, "TOPLEFT", 0, 0);
		SuperPetFrameDebuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( index == (numFirstRowBuffs+1) ) then
		buff:SetPoint("TOPLEFT", getglobal(buffName..1), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameDebuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( hasTargetofSuperPet and index == (2*numFirstRowBuffs+1) ) then
		buff:SetPoint("TOPLEFT", getglobal(buffName..(numFirstRowBuffs+1)), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameDebuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	elseif ( (index > numFirstRowBuffs) and (mod(index+(l.TARGET_DEBUFFS_PER_ROW-numFirstRowBuffs), l.TARGET_DEBUFFS_PER_ROW) == 1) and not hasTargetofSuperPet ) then
		-- Make a new row
		buff:SetPoint("TOPLEFT", getglobal(buffName..(index-l.TARGET_DEBUFFS_PER_ROW)), "BOTTOMLEFT", 0, -SuperPetFrame.buffSpacing);
		SuperPetFrameDebuffs:SetPoint("BOTTOMLEFT", buff, "BOTTOMLEFT", 0, 0);
		SuperPetFrame.buffRows = SuperPetFrame.buffRows+1;
	else
		-- Just anchor to previous
		buff:SetPoint("TOPLEFT", getglobal(buffName..(index-1)), "TOPRIGHT", offset, 0);
	end
	
	-- Resize
	buff:SetWidth(buffSize);
	buff:SetHeight(buffSize);
	local debuffFrame = getglobal(buffName..index.."Border");
	debuffFrame:SetWidth(buffSize+2);
	debuffFrame:SetHeight(buffSize+2);
end

function SuperPetFrame_HealthUpdate(elapsed, unit)
	if ( UnitIsPlayer(unit) ) then
		if ( (this.unitHPPercent > 0) and (this.unitHPPercent <= 0.2) ) then
			local alpha = 255;
			local counter = this.statusCounter + elapsed;
			local sign    = this.statusSign;
	
			if ( counter > 0.5 ) then
				sign = -sign;
				this.statusSign = sign;
			end
			counter = mod(counter, 0.5);
			this.statusCounter = counter;
	
			if ( sign == 1 ) then
				alpha = (127  + (counter * 256)) / 255;
			else
				alpha = (255 - (counter * 256)) / 255;
			end
			SuperPetPortrait:SetAlpha(alpha);
		end
	end
end

function SuperPetHealthCheck()
	if ( UnitIsPlayer("pet") ) then
		local unitHPMin, unitHPMax, unitCurrHP;
		unitHPMin, unitHPMax = this:GetMinMaxValues();
		unitCurrHP = this:GetValue();
		this:GetParent().unitHPPercent = unitCurrHP / unitHPMax;
		if ( UnitIsDead("pet") ) then
			SuperPetPortrait:SetVertexColor(0.35, 0.35, 0.35, 1.0);
		elseif ( UnitIsGhost("pet") ) then
			SuperPetPortrait:SetVertexColor(0.2, 0.2, 0.75, 1.0);
		elseif ( (this:GetParent().unitHPPercent > 0) and (this:GetParent().unitHPPercent <= 0.2) ) then
			SuperPetPortrait:SetVertexColor(1.0, 0.0, 0.0);
		else
			SuperPetPortrait:SetVertexColor(1.0, 1.0, 1.0, 1.0);
		end
	end
end

function SuperPetFrameDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, SuperPetFrameDropDown_Initialize, "MENU");
end

function SuperPetFrameDropDown_Initialize()
	UnitPopup_ShowMenu(SuperPetFrameDropDown, "PET", "pet");
end


function TargetofSuperPet_OnLoad()
	UnitFrame_Initialize("pettarget", TargetofSuperPetName, TargetofSuperPetPortrait, TargetofSuperPetHealthBar, TargetofSuperPetHealthBarText, TargetofSuperPetManaBar, TargetofSuperPetFrameManaBarText);
	SetTextStatusBarTextZeroText(TargetofSuperPetHealthBar, TEXT(DEAD));
	this:RegisterEvent("UNIT_AURA");

	SecureUnitButton_OnLoad(this, "pettarget");
	RegisterUnitWatch(TargetofSuperPetFrame);  
    ClickCastFrames = ClickCastFrames or { };
    ClickCastFrames[this] = true;
end

function TargetofSuperPet_OnHide()
	SuperPetDebuffButton_Update();
end

function TargetofSuperPet_Update()
	if ( TargetofSuperPetFrame:IsShown() ) then
		UnitFrame_Update();
		TargetofSuperPet_CheckDead();
		TargetofSuperPetHealthCheck();
		RefreshBuffs(TargetofSuperPetFrame, 0, "pettarget");
	end
end

function TargetofSuperPet_CheckDead()
	if ( (UnitHealth("pettarget") <= 0) and UnitIsConnected("pettarget") ) then
		TargetofSuperPetBackground:SetAlpha(0.9);
		TargetofSuperPetDeadText:Show();
	else
		TargetofSuperPetBackground:SetAlpha(1);
		TargetofSuperPetDeadText:Hide();
	end
end

function TargetofSuperPetHealthCheck()
	if ( UnitIsPlayer("pettarget") ) then
		local unitHPMin, unitHPMax, unitCurrHP;
		unitHPMin, unitHPMax = TargetofSuperPetHealthBar:GetMinMaxValues();
		unitCurrHP = TargetofSuperPetHealthBar:GetValue();
		TargetofSuperPetFrame.unitHPPercent = unitCurrHP / unitHPMax;
		if ( UnitIsDead("pettarget") ) then
			TargetofSuperPetPortrait:SetVertexColor(0.35, 0.35, 0.35, 1.0);
		elseif ( UnitIsGhost("pettarget") ) then
			TargetofSuperPetPortrait:SetVertexColor(0.2, 0.2, 0.75, 1.0);
		elseif ( (TargetofSuperPetFrame.unitHPPercent > 0) and (TargetofSuperPetFrame.unitHPPercent <= 0.2) ) then
			TargetofSuperPetPortrait:SetVertexColor(1.0, 0.0, 0.0);
		else
			TargetofSuperPetPortrait:SetVertexColor(1.0, 1.0, 1.0, 1.0);
		end
	end
end


function SetSuperPetSpellbarAspect()
	local frameText = getglobal(SuperPetFrameSpellBar:GetName().."Text");
	if ( frameText ) then
		frameText:SetTextHeight(10);
		frameText:ClearAllPoints();
		frameText:SetPoint("TOP", SuperPetFrameSpellBar, "TOP", 0, 4);
	end

	local frameBorder = getglobal(SuperPetFrameSpellBar:GetName().."Border");
	if ( frameBorder ) then
		frameBorder:SetTexture("Interface\\CastingBar\\UI-CastingBar-Border-Small");
		frameBorder:SetWidth(197);
		frameBorder:SetHeight(49);
		frameBorder:ClearAllPoints();
		frameBorder:SetPoint("TOP", SuperPetFrameSpellBar, "TOP", 0, 20);
	end

	local frameFlash = getglobal(SuperPetFrameSpellBar:GetName().."Flash");
	if ( frameFlash ) then
		frameFlash:SetTexture("Interface\\CastingBar\\UI-CastingBar-Flash-Small");
		frameFlash:SetWidth(197);
		frameFlash:SetHeight(49);
		frameFlash:ClearAllPoints();
		frameFlash:SetPoint("TOP", SuperPetFrameSpellBar, "TOP", 0, 20);
	end
end

function SuperPet_Spellbar_OnLoad()
	this:RegisterEvent("CVAR_UPDATE");
	
	CastingBarFrame_OnLoad("pet", false);

	local barIcon = getglobal(this:GetName().."Icon");
	barIcon:Show();

	SetSuperPetSpellbarAspect();
	
	-- check to see if the castbar should be shown
	if ( GetCVar( UIOptionsFrameCheckButtons["SHOW_TARGET_CASTBAR"].cvar ) == "0") then
		this.showCastbar = false;	
	end
end

function SuperPet_Spellbar_OnEvent()
	local newevent = event;
	local newarg1 = arg1;
	--	Check for target specific events
	if ( (event == "CVAR_UPDATE") and (arg1 == "SHOW_TARGET_CASTBAR") ) then
		if ( GetCVar( UIOptionsFrameCheckButtons[arg1].cvar ) == "0") then
			this.showCastbar = false;
		else
			this.showCastbar = true;
		end
		
		if ( not this.showCastbar ) then
			this:Hide();
		elseif ( this.casting or this.channeling ) then
			this:Show();
		end
		return;
	end

	CastingBarFrame_OnEvent(newevent, newarg1);
end

function SuperPet_Spellbar_AdjustPosition()
	local yPos = 5;
	if ( SuperPetFrame.buffRows and SuperPetFrame.buffRows <= 2 ) then
		yPos = 38;
	elseif ( SuperPetFrame.buffRows ) then
		yPos = 19 * SuperPetFrame.buffRows;
	end
	if ( TargetofSuperPetFrame:IsShown() ) then
		if ( yPos <= 25 ) then
			yPos = yPos + 25;
		end
	else
		yPos = yPos - 5;
	end
	SuperPetFrameSpellBar:SetPoint("BOTTOM", "SuperPetFrame", "BOTTOM", -15, -yPos);
end

function SuperPetFrameHealthBarText_UpdateTextString(textStatusBar)
	if ( not textStatusBar ) then
		textStatusBar = this;
	end
	local string = SuperPetFrameHealthBarText;
		local value = textStatusBar:GetValue();
		local valueMin, valueMax = textStatusBar:GetMinMaxValues();
		if ( valueMax > 0 ) then
			if (MobHealthDB) then
				-- No longer use default health bar text functions.
				SuperPetFrameHealthBar.TextString = nil;

				if (not UnitIsPlayer("pet") and not UnitIsUnit("pet", "pet")) then
					local petName,petLevel = UnitName("pet"),UnitLevel("pet");
					if (petName == nil or petLevel == nil) then
						return;
					end
					local ppp = MobHealth_PPP(petName..":"..petLevel);
					local curHP = math.floor(value * ppp + 0.5);
					local maxHP = math.floor(100 * ppp + 0.5);
					if (ppp and ppp ~= 0) then
					   string:SetText(curHP.." / "..maxHP);
					else
					   string:SetText(value.."%");
					end
				else
					string:SetText(value.." / "..valueMax);
				end
				string:Show();
			end
		end
end

function SuperPetFrameHealthBar_OnValueChanged(value)
	SuperPetFrameHealthBarText_UpdateTextString();
	HealthBar_OnValueChanged(value);
end

function SuperPetFrame_OnDragStart()
	if (not SuperPetFrameOptions.lockpos) then
		this:GetParent():StartMoving();
		this.isMoving = true;
	end
end

function SuperPetFrame_OnDragStop()
	 this:GetParent():StopMovingOrSizing();
	 this.isMoving = false;
end

function SuperPetFrame_OnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	if (not SuperPetFrameOptions.lockpos) then
		GameTooltip:SetText(SUPERPETFRAME_DRAG, nil, nil, nil, nil, 1);
	else
		GameTooltip:SetText(SUPERPETFRAME_DRAG_LOCKED, nil, nil, nil, nil, 1);
	end
end

function SuperPetFrame_CheckHappiness()
	local happiness, damagePercentage, loyaltyRate = GetPetHappiness();
	SuperPetFrameHappiness:Show();
	if ( happiness == 1 ) then
		SuperPetFrameHappinessTexture:SetTexCoord(0.375, 0.5625, 0, 0.359375);
	elseif ( happiness == 2 ) then
		SuperPetFrameHappinessTexture:SetTexCoord(0.1875, 0.375, 0, 0.359375);
	elseif ( happiness == 3 ) then
		SuperPetFrameHappinessTexture:SetTexCoord(0, 0.1875, 0, 0.359375);
	else
		happiness = 1;
	end
	SuperPetFrameHappiness.tooltip = getglobal("PET_HAPPINESS"..happiness);
	SuperPetFrameHappiness.tooltipDamage = format(PET_DAMAGE_PERCENTAGE, damagePercentage);
	if ( loyaltyRate < 0 ) then
		SuperPetFrameHappiness.tooltipLoyalty = getglobal("LOSING_LOYALTY");
	elseif ( loyaltyRate > 0 ) then
		SuperPetFrameHappiness.tooltipLoyalty = getglobal("GAINING_LOYALTY");
	else
		SuperPetFrameHappiness.tooltipLoyalty = nil;
	end
end
